{% extends 'netjsongraph/base.html' %}
{% load static %}
{% block title %}{{ topology.label }}{% endblock %}
{% block script %}
    {{ block.super }}
    <script src="{% static 'admin/js/jquery.init.js' %}"></script>
    <script src="{% static 'netjsongraph/js/lib/jquery-ui.min.js' %}"></script>
    {% include './netjsongraph-script.html' %}
    <div class="djnjg-overlay"></div>
    <script>
      const graph = window.loadNetJsonGraph('.djnjg-overlay');
    </script>
{% endblock %}
