#legend h4 {
    margin: 10px 0;
    text-align: center;
}

#legend {
    position: absolute;
    right: 25px;
    bottom: 25px;
    width: auto;
    height: auto;
    max-width: 250px;
    padding: 8px 15px;
    background: #fbfbfb;
    border-radius: 8px;
    color: #000;
    font-family: Arial, sans-serif;
    font-family: sans-serif;
    font-size: 14px;
    z-index: 2;
    user-select: text;
    transition: all 0.5s;
}

.open {
    left: 370px !important;
}

#legend p {
    margin: 10px 0;
    display: flex;
    align-items: center;
}
#legend span {
    width: 16px;
    margin-right: 5px;
}
#legend .link-up,
#legend .link-down {
    display: inline-block;
    height: 5px;
    border-bottom-width: 6px;
    border-bottom-style: solid;
}
#legend .link-up {
    color: #3acc38;
    margin-right: 10px;
}
#legend .link-down {
    color: red;
    margin-right: 10px;
}

.njg-sideBar h2 {
    color: #000;
    margin: 0;
    font-size: 20px;
}

.njg-sideBar * {
    color: #000;
    box-sizing: border-box;
}

.djnjg-overlay {
    position: fixed !important;
    left: 0;
    top: 0;
    z-index: 10000;
    width: 100%;
    height: 100%;
    background: rgba(0, 0, 0, 0.95);
}

.djnjg-overlay .closeBtn {
    border: none;
    outline: none;
    background: rgba(217, 79, 52, 0.85);
    padding: 0 8px;
    border-radius: 5px;
    position: absolute;
    right: 15px;
    top: 15px;
    font-size: 30px;
    color: #fff;
    cursor: pointer;
}

body .frontend {
    overflow: hidden;
    font-family: Arial, sans-serif;
    line-height: 20px;
    background-color: #2b2b2b;
}

.njg-switcher {
    background: #fff;
    border-radius: 5px;
    font-family: Arial, sans-serif;
    font-family: sans-serif;
    font-size: 14px;
    line-height: 20px;
    height: auto;
    max-width: 400px;
    min-width: auto;
    position: absolute;
    left: 25px;
    bottom: 25px;
    padding: 5px 8px;
    transition: all 0.5s;
}

.njg-switcher * {
    color: #000;
}

#njg-datepicker {
    margin: 0 0 0 5px !important;
    font-size: 13px;
    padding: 5px 10px;
    width: 94px;
    outline: none;
    color: #000 !important;
}

#njg-datepicker:focus {
    border: 1px solid #df5d43;
    color: #df5d43 !important;
}

#ui-datepicker-div {
    font-size: 15px;
    line-height: initial;
    margin-top: -13px !important;
    margin-left: -45px !important;
    border-radius: 5px !important;
}

#ui-datepicker-div tr {
    background: #fff !important;
}

.ui-datepicker-header {
    background: #fff !important;
    color: #000 !important;
    border: none !important;
}

.ui-state-default {
    border: none !important;
    background: #fff !important;
    color: #000 !important;
    text-align: center !important;
}

.ui-state-default:hover,
.ui-state-default:focus,
.ui-state-active {
    background: #333 !important;
    color: #fff !important;
}

.ui-state-default:hover {
    background: #7b7b7b !important;
}

.ui-datepicker-today a {
    background: #df5d43 !important;
    color: #fff !important;
}

.ui-datepicker thead th {
    line-height: normal;
    text-transform: uppercase;
    font-size: 11px;
    background: #fff;
    color: #000;
}
