# Generated by Django 3.2.18 on 2023-04-17 18:06

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.TOPOLOGY_TOPOLOGY_MODEL),
        ('topology_device', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='WifiMesh',
            fields=[
                (
                    'id',
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ('mesh_id', models.CharField(max_length=32, verbose_name='Mesh ID')),
                (
                    'topology',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.TOPOLOGY_TOPOLOGY_MODEL,
                    ),
                ),
            ],
            options={
                'abstract': False,
                'swappable': 'TOPOLOGY_DEVICE_WIFIMESH_MODEL',
            },
        ),
    ]
